import { DeleteOutline, FileOpenOutlined } from "@mui/icons-material";
import { Box, Button, Stack, TextField } from "@mui/material";
import { open } from "@tauri-apps/api/dialog";
import { convertFileSrc, invoke } from "@tauri-apps/api/tauri";
import { useAtom } from "jotai";
import Image from "next/image";
import { FC, useEffect, useState } from "react";
import { useSettings } from "../../../../contexts/SettingsContext";
import { stagedSettingsAtom } from "../../../../lib/store/jotai/settings";
import { CategoryTitle } from "../CategoryTitle";
import { SettingsItem } from "../SettingsItem";

interface SettingsTabBackgroundProps {
  sx?: any;
}

export const SettingsTabBackground: FC<SettingsTabBackgroundProps> = ({ sx }) => {
  // contexts
  const { settings, updateSettings } = useSettings();

  // atoms
  const [stagedSettings, setStagedSettings] = useAtom(stagedSettingsAtom);

  // states
  const [oldWallpaperPath, setOldWallpaperPath] = useState<string | null>(null);
  const [targetWallpaperPath, setTargetWallpaperPath] = useState<string | null>(null);
  const [imageUrl, setImageUrl] = useState<string | null>(null);

  const handleSettingsBackgroundValueChange = (
    settingKey: string,
    settingValue: boolean | number | string | number[],
  ) => {
    const newSettings = {
      ...stagedSettings,
      background: {
        ...stagedSettings.background,
        [settingKey]: settingValue,
      },
    };
    setStagedSettings(newSettings);

    return newSettings;
  };

  const setImageSrc = async (filePath: string) => {
    const assetUrl = convertFileSrc(filePath);
    setImageUrl(assetUrl);
  };

  const selectImage = async () => {
    const { appLocalDataDir, basename } = await import("@tauri-apps/api/path");

    let selectedFilePath = await open({
      multiple: false,
      filters: [
        {
          name: "Images",
          extensions: ["jpg", "png", "jpeg", "webp", "gif"],
        },
      ],
    });

    // if the user somehow manages to select multiple files, take the first file
    if (Array.isArray(selectedFilePath)) {
      selectedFilePath = selectedFilePath[0];
    }

    if (selectedFilePath) {
      setTargetWallpaperPath(selectedFilePath);

      // construct the destination file path
      const appLocalDataDirPath = await appLocalDataDir();
      const filename = await basename(selectedFilePath);
      const destinationFilePath = appLocalDataDirPath + filename;
      handleSettingsBackgroundValueChange("background_image_path", destinationFilePath);
    }
  };

  const clearImage = async () => {
    if (stagedSettings.background.background_image_path) {
      try {
        await invoke("delete_old_wallpaper_image", {
          filePathString: stagedSettings.background.background_image_path,
        });
      } catch (error) {
        console.error("Failed to delete old wallpaper image", error);
      }
    }

    const newSettings = handleSettingsBackgroundValueChange("background_image_path", "");

    updateSettings(newSettings);
  };

  // if settings.background.background_image_path changes, update the image
  useEffect(() => {
    const applyWallpaper = async () => {
      try {
        await invoke("process_wallpaper_image", {
          filePathString: targetWallpaperPath,
        });
        // if there is already a wallpaper file, delete it
        if (settings.background.background_image_path) {
          try {
            await invoke("delete_old_wallpaper_image", {
              filePathString: oldWallpaperPath,
            });
          } catch (error) {
            console.error("Failed to delete old wallpaper image", error);
          }
        }
      } catch (error) {
        console.error(error);
      }
    };

    applyWallpaper();

    setOldWallpaperPath(settings.background.background_image_path);
  }, [settings.background.background_image_path]);

  // update the preview image when stagedSettings.background.background_image_path changes
  // useEffect(() => {
  //   if (stagedSettings.background.background_image_path) {
  //     setImageSrc(stagedSettings.background.background_image_path);
  //   } else {
  //     setImageUrl(null);
  //   }
  // }, [stagedSettings.background.background_image_path]);

  return (
    <Box sx={{ sx }}>
      <CategoryTitle title="Wallpaper" />
      <Box
        sx={{
          position: "relative",
          // dynamic width based on the parent container
          width: "100%",
          // 3:2 aspect ratio (2/3 = 66.67%) qwuik mafs
          paddingBottom: "66.67%",
          // Hide overflow to maintain aspect ratio
          overflow: "hidden",
          borderRadius: "8px", // Optional: rounded corners
        }}
      >
        <Image
          src={imageUrl || "oh no"}
          alt="Image not found"
          // fill the box r/catsareliquid
          layout="fill"
          objectFit="cover"
        />
      </Box>
      <Box
        sx={{
          mb: 2,
          mt: 1,
        }}
      >
        <Stack
          direction="row"
          spacing={1}
        >
          <Button
            color="primary"
            onClick={() => {
              selectImage().then(() => updateSettings(stagedSettings));
            }}
            startIcon={<FileOpenOutlined />}
            size="small"
            variant="outlined"
          >
            Select
          </Button>
          <Button
            color="warning"
            onClick={clearImage}
            startIcon={<DeleteOutline />}
            size="small"
            variant="outlined"
          >
            Clear
          </Button>
        </Stack>
      </Box>
      <CategoryTitle title="Colors" />
      <SettingsItem
        defaultText="#202124"
        description="Background color"
        input={
          <TextField
            name="background_color"
            onChange={(e) => {
              handleSettingsBackgroundValueChange(e.target.name, e.target.value);
            }}
            sx={{
              width: "100%",
            }}
            size="small"
            type="color"
            value={stagedSettings.background.background_color}
            variant="standard"
          />
        }
      />
      <button
        onClick={() => {
          updateSettings(stagedSettings);
        }}
      >
        apply
      </button>
    </Box>
  );
};
